(*******************************************************************************
tBmToolBar v1.0

Derived from
    tCustomPanel

Properties
    PopupCaption : Caption for popup menu (used for configure toolbar)

Inherited properties

Events

Methods
    ConfigToolbar     : Used to show manually configuration form
    ToggleBtnSizes    : Switch button sizes : Big (32) / Small (16)
    ToggleShowCaption : Switch button caption : true / false


********************************************************************************
Author name=BOURMAD Mehdi
Author E-mail=bourmad@mygale.org
Author URL=www.mygale.org/~bourmad
*******************************************************************************)
unit BmTBar;

interface

uses
  WinTypes, WinProcs, Messages,
  Buttons, Classes, Controls, Dialogs, DsgnIntf, ExtCtrls, Forms, Graphics,
  IniFiles, Menus, StdCtrls, SysUtils,
  BmTbMisc, BmTbForm,
  ExplBtn;


type
  tBmToolBar = class ( TCustomPanel)
  private
    FAbout        : TAboutMeProperty;
    FBtnSize      : tTbBtnSize;
    FPopup        : TPopupMenu;
    FPopupCaption : String;
    FShowCaption  : Boolean;
    FTbEditor     : tBmTbEditor;
  protected
    procedure Loaded; override;
    procedure MoveBut (Sender: TObject; Button: TMouseButton;
                       Shift: TShiftState; X, Y: Integer);
    function CreateNewButton (aBtnInfo : tButtonInfo; X : Integer) : tTbButton;
    procedure ResizeButtons;
    procedure OrderButton (aButton : tTbButton);
    function FindBtnNumber  (aNumber : integer) : tTbButton;
    function FindBtnCaption (aCaption : string) : tTbButton;
    procedure PlaceButtons;
    procedure RefreshSize;
    procedure RefreshCaption;
    procedure DragDrop (Sender, Source: TObject; X,Y: Integer);
    procedure DragOver (Sender, Source: TObject; X, Y: Integer;
                        State: TDragState; var Accept: Boolean);
    procedure EndDrag (Sender, Target: TObject; X, Y: Integer);
    procedure SetBtnSize (value : tTbBtnSize);
    procedure SetShowCaption (value : boolean);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ConfigToolbar (Sender : tObject);
    procedure DisableButton (aCaption : String);
    procedure EnableButton (aCaption : String);
    procedure ToggleBtnSizes (Sender : tObject);
    procedure ToggleShowCaption (Sender : tObject);
    procedure AddButton2List (aCaption :String; aBitmap16, aBitmap32 : tBitmap; OnClick : tNotifyEvent);
    procedure AddBtnRes2List (aCaption, Res16, Res32 :String; OnClick : tNotifyEvent);
    procedure SaveToFile (aFile : String);
    procedure LoadFromFile (aFile : String);
  published
    property About : TAboutMeProperty read FAbout write FAbout;
    property BtnSize : tTbBtnSize read FBtnSize write SetBtnSize default tbBtn32;
    property PopupCaption : String read FPopupCaption write FPopupCaption;
    property ShowCaption : boolean read FShowCaption write SetShowCaption default true;
    property Align;
    property Font;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderStyle;
    property Visible;
  end;

implementation

constructor tBmToolBar.Create(AOwner: TComponent);
begin
  inherited Create (aOwner);

  {Set Global variables to startup defaults}
  BmTB_ConfigTB := false;
  BmTB_Working := false;

  OnDragDrop := DragDrop;
  OnDragOver := DragOver;
  OnEndDrag  := EndDrag;

  FBtnSize := tbBtn32;
  case FBtnSize of
    tbBtn32 : begin
                Height := BtnSize32+BtnTop*2;
              end;
    tbBtn16 : begin
                Height := BtnSize16+BtnTop*2;
              end;
  end;

  FPopup := tPopupMenu.Create (Self);
  PopupMenu := FPopup;

  FPopupCaption := '';

  FShowCaption := true;

  FTbEditor := TBmTbEditor.Create(Self);
  FTbEditor.Visible := false;
end;

destructor tBmToolBar.Destroy;
begin
  FPopup.Free;
  FTbEditor.Free;
  Inherited Destroy;
end;


procedure tBmToolBar.Loaded;
var
  NewItem: TMenuItem;
begin
  inherited Loaded;
  Caption := '';

  NewItem := TMenuItem.Create(FPopup);
  NewItem.Caption := FPopupCaption;
  FPopup.Items.Add(NewItem);

  NewItem := TMenuItem.Create(FPopup);
  NewItem.Caption := '-';
  FPopup.Items.Add(NewItem);

  NewItem := TMenuItem.Create(FPopup);
  NewItem.Caption := IdsConfigure;
  NewItem.OnClick := ConfigToolbar;
  FPopup.Items.Add(NewItem);

  NewItem := TMenuItem.Create(FPopup);
  NewItem.Caption := '-';
  FPopup.Items.Add(NewItem);

  NewItem := TMenuItem.Create(FPopup);
  NewItem.Caption := IdsBigBtn;
  NewItem.Checked := FBtnSize = tbBtn32;
  NewItem.OnClick := ToggleBtnSizes;
  FPopup.Items.Add(NewItem);

  NewItem := TMenuItem.Create(FPopup);
  NewItem.Caption := IdsShCaption;
  NewItem.Checked := FShowCaption;
  NewItem.OnClick := ToggleShowCaption;
  FPopup.Items.Add(NewItem);
end;

{This method, assigned to the OnMouseDown event of new buttons,
 kicks in when we click on an existing button on the toolbar}
procedure tBmToolBar.MoveBut (Sender: TObject; Button: TMouseButton;
                              Shift: TShiftState; X, Y: Integer);
var
  ClickEvent : tNotifyEvent;
begin
  BmTB_SendFromPool := false; {this button didn't come from Editor Window}
  if BmTB_ConfigTB = True then{if configuring toolbar when click this button...}
  begin
  BmTB_Working := true;       {put us in "just moving existing button" mode}
  With sender as tTbButton do
  begin
    ClickEvent := (Sender as tTbButton).OnClick;
    (Sender as tTbButton).OnClick := nil;
    beginDrag(false);
    (Sender as tTbButton).OnClick := ClickEvent;
  end;
  end;
end;

function tBmToolBar.CreateNewButton (aBtnInfo : tButtonInfo; X : Integer) : tTbButton;
var
  aButton : tTbButton;
begin
  aButton := tTbButton.Create(Self);  {create the new button in memory}
  aButton.Parent := Self;             {set the new buttons' parent}
  aButton.Caption := aBtnInfo.Caption;
  case FBtnSize of
    tbBtn32 : aButton.Bitmap := aBtnInfo.Bitmap32;
    tbBtn16 : aButton.Bitmap := aBtnInfo.Bitmap16;
  end;
  aButton.Layout := blBitmapLeft;
  aButton.Top := BtnTop;              {assign a default Y pos}
  aButton.Left := X;        {assign left of the button to drop's X position}
  aButton.Visible := true;
  aButton.OnMouseDown := MoveBut;     {assign to the OnMouseDown event}
  aButton.OnClick := aBtnInfo.OnClick;
  case FBtnSize of
    tbBtn32 : aButton.Height := BtnSize32;
    tbBtn16 : aButton.Height := BtnSize16;
  end;
  if aButton.Bitmap<>nil
  then begin
    if FShowCaption
      then aButton.Width :=
             aButton.Bitmap.Width + Canvas.TextWidth (aButton.Caption)+BtnMarginC
      else aButton.Width := aButton.Height;
  end
  else begin
    aButton.Width := Canvas.TextWidth (aButton.Caption)+BtnMargin*2;
  end;
  OrderButton (aButton);
  Result := aButton;
end;

procedure tBmToolBar.ResizeButtons;
var
  i : integer;
  aButton : tTbButton;
begin
  for i := 0 to ControlCount-1 do
  begin
    if (Controls[i] is tTbButton)
    then begin
      aButton := Controls[i] as tTbButton;
      case FBtnSize of
        tbBtn32 : begin
                    aButton.Height := BtnSize32;
                  end;
        tbBtn16 : begin
                    aButton.Height := BtnSize16;
                  end;
      end;
      if FShowCaption
        then aButton.Width := aButton.Bitmap.Width
                              + Canvas.TextWidth (aButton.Caption)+BtnMarginC
        else aButton.Width := aButton.Height;
    end;
  end;
end;

procedure tBmToolBar.OrderButton (aButton : tTbButton);
var
  aButton2 : tTbButton;
  i        : integer;
begin
  {Need to check each buttons because Controls[i] isn't ordered by TabOrder}
  for i:=0 to ControlCount-1 do
  begin
    if (Controls[i] is tTbButton)
    then begin
      aButton2 := Controls[i] as tTbButton;
      if (aButton.Left < aButton2.Left)
      and (aButton.TabOrder > aButton2.TabOrder)
        then aButton.TabOrder := aButton2.TabOrder;
    end;
  end;
end;

function tBmToolBar.FindBtnNumber (aNumber : integer) : tTbButton;
var
  i : integer;
begin
  Result := nil;
  for i := 0 to ControlCount-1 do
  begin
    if (Controls[i] is tTbButton)
    then begin
      if (Controls[i] as tTbButton).TabOrder = aNumber
        then Result := (Controls[i] as tTbButton);
    end;
  end;
end;

function tBmToolBar.FindBtnCaption (aCaption : string) : tTbButton;
var
  i : integer;
begin
  Result := nil;
  for i := 0 to ControlCount-1 do
  begin
    if (Controls[i] is tTbButton)
    then begin
      if (Controls[i] as tTbButton).Caption = aCaption
        then Result := (Controls[i] as tTbButton);
    end;
  end;
end;


procedure tBmToolBar.PlaceButtons;
var
  aButton : tTbButton;
  left,
  i       : integer;
begin
  ResizeButtons;
  Left := ToolbarMargin;
  for i := 0 to ControlCount-1 do
  begin
    aButton := FindBtnNumber(i);
    aButton.Left := Left;
    Left := Left + aButton.Width + BtnSpace;
  end;
end;

procedure tBmToolBar.RefreshSize;
var
  i, j     : integer;
  aButton  : tTbButton;
  aCaption : String;
  BtnInfo  : tButtonInfo;
begin
  for i := 0 to ControlCount-1 do
  begin
    if (Controls[i] is tTbButton)
    then begin
      aButton := Controls[i] as tTbButton;
      aCaption := aButton.Caption;
      J := FTbEditor.ListBoxBtn.Items.IndexOf (aCaption);
      BtnInfo := FTbEditor.ListBoxBtn.Items.Objects[J] as tButtonInfo;
      case FBtnSize of
        tbBtn32 : begin
                    aButton.Bitmap := BtnInfo.Bitmap32;
                  end;
        tbBtn16 : begin
                    aButton.Bitmap := BtnInfo.Bitmap16;
                  end;
      end;
    end;
  end;
  case FBtnSize of
    tbBtn32 : begin
                Height := BtnSize32+BtnTop*2;
              end;
    tbBtn16 : begin
                Height := BtnSize16+BtnTop*2;
              end;
  end;
  PlaceButtons;
end;

procedure tBmToolBar.RefreshCaption;
var
  i, j     : integer;
  aButton  : tTbButton;
  aCaption : String;
  BtnInfo  : tButtonInfo;
begin
  for i := 0 to ControlCount-1 do
  begin
    if (Controls[i] is tTbButton)
    then begin
      aButton := Controls[i] as tTbButton;
      aCaption := aButton.Caption;
      infodebug (aCaption+' '+inttostr(aButton.TabOrder)+#13#10);
      J := FTbEditor.ListBoxBtn.Items.IndexOf (aCaption);
      BtnInfo := FTbEditor.ListBoxBtn.Items.Objects[J] as tButtonInfo;
      if FShowCaption
      then begin
        aButton.Options := aButton.Options - [boIconOnly];
      end
      else begin
        aButton.Options := aButton.Options + [boIconOnly];
      end;
    end;
  end;
  PlaceButtons;
end;

{This Method handles the action when a button is dropped onto the toolbar.
It has 2 ways to go:
  #1: If it is a brand new button (BmTB_Working = false), then assign the stuff
      listed below from the button in the Editor Window to the new button, and
  #2: If we're just moving and existing button (BmTB_Working = true),
      just move it to the new location we dragged it to.
Also, if you attempt to drag a button onto the toolbar that is already on the
toolbar, a ECOMPONENTERROR exception is generated.  We handle that here, as well}
procedure tBmToolBar.DragDrop (Sender, Source: TObject; X,Y: Integer);
var
  aButton    : tTbButton;
  ListBox    : tListBox;
  ButtonInfo : tButtonInfo;
begin
  if Source is tListBox
  then begin
    if (BmTB_Working = false) {if this is a brand new button...}
    then begin
      ListBox := Source as tListBox;
      ButtonInfo := ListBox.Items.Objects[ListBox.ItemIndex] as tButtonInfo;
      CreateNewButton (ButtonInfo, X);
      PlaceButtons;
    end;
  end;
  if Source is tTbButton then
  begin
    if (BmTB_Working = true) {if we're just moving an existing button...}
    then begin
      aButton := Source as tTbButton;
      aButton.Top := BtnTop;            {set it to these coordinates}
      aButton.Left := x;
      OrderButton (aButton);
      PlaceButtons;
    end;
  end;
  BmTB_Working := false; {make sure don't stay in "moving existing button" mode}
end;

{If we're dragging a component from Toolbar editor over the Toolbar,
we like that and are willing to accept it!}
procedure tBmToolBar.DragOver (Sender, Source: TObject; X, Y: Integer;
                               State: TDragState; var Accept: Boolean);
var
  i : integer;
  accepted : boolean;
  ListBox : tListBox;
  ButtonInfo : tButtonInfo;
begin
  if (source as tControl).Parent is tBmTbEditor
  then begin
    Accepted := true;
    ListBox := Source as tListBox;
    ButtonInfo := ListBox.Items.Objects[ListBox.ItemIndex] as tButtonInfo;
    for i := 0 to ControlCount-1 do {search if the button exists in toolbar}
    begin
      if (Controls[i] is tTbButton)
      and ((Controls[i] as tTbButton).Caption = ButtonInfo.Caption)
        then accepted := false;
    end;
    Accept := Accepted;
  end
  else if (source as tControl).Parent = Self
    then Accept := true
    else Accept := False;
end;

{This method is a precautionary step,
to make sure we can't delete buttons from the Editor}
procedure tBmToolBar.EndDrag (Sender, Target: TObject; X, Y: Integer);
begin
  BmTB_Working := false;      {set to "new button" mode}
  BmTB_SendFromPool := true;  {set to "button from editor" mode}
end;

procedure tBmToolBar.SetBtnSize (value : tTbBtnSize);
begin
  if value <> FBtnSize
  then begin
    FBtnSize := value;
    RefreshSize;
  end;
end;

procedure tBmToolBar.SetShowCaption (value : boolean);
begin
  if value <> FShowCaption
  then begin
    FShowCaption := value;
    RefreshCaption;
  end;
end;

procedure tBmToolBar.ConfigToolbar (Sender : tObject);
begin
  try
    try
      FTbEditor.Show;
    except
      FTbEditor.Close;
    end;
  except
    FTbEditor.Close;
  end;
end;

procedure tBmToolBar.DisableButton (aCaption : String);
var
  aButton : tTbButton;
begin
  aButton := FindBtnCaption (aCaption);
  aButton.Enabled := false;
end;

procedure tBmToolBar.EnableButton (aCaption : String);
var
  aButton : tTbButton;
begin
  aButton := FindBtnCaption (aCaption);
  aButton.Enabled := true;
end;

procedure tBmToolBar.ToggleBtnSizes (Sender : tObject);
begin
  (Sender as tMenuItem).Checked := not (Sender as tMenuItem).Checked;
  case FBtnSize of
    tbBtn32 : SetBtnSize (tbBtn16);
    tbBtn16 : SetBtnSize (tbBtn32);
  end;
end;

procedure tBmToolBar.ToggleShowCaption (Sender : tObject);
begin
  (Sender as tMenuItem).Checked := not (Sender as tMenuItem).Checked;
  SetShowCaption (not FShowCaption);
end;

procedure tBmToolBar.AddButton2List (aCaption :String;
                                     aBitmap16, aBitmap32 : tBitmap;
                                     OnClick : tNotifyEvent);
begin
  FTbEditor.AddButton2List (aCaption, aBitmap16, aBitmap32, OnClick);
end;

procedure tBmToolBar.AddBtnRes2List (aCaption, Res16, Res32 :String; OnClick : tNotifyEvent);
begin
  FTbEditor.AddBtnRes2List (aCaption, Res16, Res32, OnClick);
end;

procedure tBmToolBar.SaveToFile (aFile : String);
var
  FicIni  : tIniFile;
  i       : integer;
  aButton : tTbButton;
begin
  FicIni := tIniFile.Create(aFile);
  FicIni.EraseSection (Self.Name);
  {Font information}
  FicIni.WriteString (Self.Name, IdsIniFontName, Font.Name);
  FicIni.WriteInteger (Self.Name, IdsIniFontSize, Font.Size);
  {Buttons information}
  FicIni.WriteBool (Self.Name, IdsIniBigBtn, FBtnSize=tbBtn32);
  FicIni.WriteBool (Self.Name, IdsIniShowCaption, FShowCaption);

  for i := 0 to ControlCount-1 do
  begin
    if (Controls[i] is tTbButton)
    then begin
      aButton := Controls[i] as tTbButton;
      FicIni.WriteInteger (Self.Name, aButton.Caption, aButton.Left);
    end;
  end;
  FicIni.Free;
end;

procedure tBmToolBar.LoadFromFile (aFile : String);
var
  FicIni   : tIniFile;
  Lines    : tStringList;
  i, j     : integer;
  BtnInfo  : tButtonInfo;
  aCaption : String;
  aLeft    : Integer;
begin
  FicIni := tIniFile.Create(aFile);
  Lines := tStringList.Create;
  FicIni.ReadSection (Self.Name, Lines);

  {Font information}
  J := Lines.IndexOf (IdsIniFontName);
  if J>=0
  then begin
    Font.Name := FicIni.ReadString (Self.Name, Lines[J], Font.Name);
    Canvas.Font.Name := Font.Name;
    Lines.Delete (J);
  end;
  J := Lines.IndexOf (IdsIniFontSize);
  if J>=0
  then begin
    Font.Size := FicIni.ReadInteger (Self.Name, Lines[J], Font.Size);
    Canvas.Font.Size := Font.Size;
    Lines.Delete (J);
  end;

  {Buttons information}
  J := Lines.IndexOf (IdsIniBigBtn);
  if J>=0
  then begin
    if FicIni.ReadBool (Self.Name, Lines[J], FBtnSize=tbBtn32)
      then FBtnSize := tbBtn32;
    for i := 0 to FPopup.Items.Count-1 do
    begin
      if FPopup.Items[I].Caption = IdsBigBtn
        then FPopup.Items[I].Checked := FBtnSize=tbBtn32;
    end;
    Lines.Delete (J);
  end;
  J := Lines.IndexOf (IdsIniShowCaption);
  if J>=0
  then begin
    FShowCaption := FicIni.ReadBool (Self.Name, Lines[J], FShowCaption);
    for i := 0 to FPopup.Items.Count-1 do
    begin
      if FPopup.Items[I].Caption = IdsShCaption
        then FPopup.Items[I].Checked := FShowCaption;
    end;
    Lines.Delete (J);
  end;

  {buttons...}
  for i := 0 to Lines.Count-1 do
  begin
    aCaption := Lines[i];
    aLeft := FicIni.ReadInteger (Self.Name, Lines[i], -1);
    J := FTbEditor.ListBoxBtn.Items.IndexOf (aCaption);
    BtnInfo := FTbEditor.ListBoxBtn.Items.Objects[J] as tButtonInfo;
    if aLeft>0
      then CreateNewButton (BtnInfo, aLeft);
  end;
  PlaceButtons;

  Lines.Free;
  FicIni.Free;
end;

end.
